import { Request, Response, NextFunction} from 'express';
import { Product } from '../schemas/productSchema';
import { createProduct, getAllProducts, getOne } from '../models/productModel';
import { ApiError } from '../utils/ApiError';

const create = async (req: Request, res: Response, next: NextFunction) => {
  try {
    const { name, description, price } = req.body;
    const product = await createProduct({
      name,
      description,
      price,
      userId: req.body.userId
    });
    if(product instanceof Error) {
      console.log("Error in create product")
      return next(product);
    }
    res.status(201).json(product);
  } catch {
    const error = new ApiError('Error during product creation');
    error.statusCode = 500;
    error.status = 'fail';
    next(error);
  }
}

const getAll = async (req: Request, res: Response) => {
  try {
    const products = await getAllProducts({
      page: Number(req.query.page),
      limit: Number(req.query.limit)
    });
    if(products instanceof Error) {
      console.log("Error in get all products")
      return res.status(products.statusCode).json({ error: products.message });
    }
    res.status(200).json(products);
  } catch {
    const error = new ApiError('Error during product creation');
    error.statusCode = 500;
    error.status = 'fail';
    res.status(error.statusCode).json({ error: error.message });
  }
}

export async function getProduct(req: Request, res: Response) {
  try {
    const product = await getOne(req.params.id);
    if(product instanceof Error) {
      return res.status(product.statusCode).json({ error: product.message });
    }
    res.status(200).json(product);
  } catch {
    const error = new ApiError('Error during product creation');
    error.statusCode = 500;
    error.status = 'fail';
    res.status(error.statusCode).json({ error: error.message });
  }
}

export {
  create,
  getAll,
  getOne
}