import { Request, Response, NextFunction } from 'express';
import { ApiError } from '../utils/ApiError';
import validate from 'deep-email-validator';

const isValidLogin = async (req: Request, res: Response, next: NextFunction) => {
  
    const user = req.body;
    if (!user.email || !user.password) {
      const error = new ApiError(`${!user.email ? 'email' : 'password'} is required`);
      error.statusCode = 400;
      error.status = 'fail';
      return next(error);
    }

    next();
}

const isValidCreateUser = async (req: Request, res: Response, next: NextFunction) => {

  const user = req.body;
  if (!user.email || !user.password || !user.address) {
    const error = new ApiError(`${!user.email ? 'email' : !user.password ? 'password' : 'address'} is required}`);
    error.statusCode = 400;
    error.status = 'fail';
    return next(error);
  }

  const { valid, reason } = await validate(user.email);
  if (!valid) {
    const error = new ApiError(`Invalid email: ${reason}`);
    error.statusCode = 400;
    error.status = 'fail';
    return next(error);
  }
  next();
}

export {
  isValidLogin,
  isValidCreateUser
}