import { User } from "../schemas/userSchema";
import { ApiError } from "../utils/ApiError";
import bcrypt from 'bcryptjs';

const createUser = async (user: any) => {

  const userExists = await User.exists({ email: user.email });
  if (userExists) {
    const error = new ApiError('User already exists, Try login :)');
    error.statusCode = 400;
    error.status = 'fail';
    return error;
  }

  const salt = await bcrypt.genSalt(10);
  user.password = await bcrypt.hash(user.password, salt);

  const newUser = new User(user);
  try {
    await newUser.save();
    return {
      email: newUser.email,
      address: newUser.address,
    };
  } catch (error) {
    return error;
  }
}

const loginUser = async (user: any) => {
  const { email, password } = user;
  const userExists = await User.findOne({ email });
  if (!userExists) {
    const error = new ApiError('Invalid email or password');
    error.statusCode = 404;
    error.status = 'fail';
    return error;
  }

  const isMatch = await bcrypt.compare(password, userExists.password);
  if(!isMatch) {
    const error = new ApiError('Invalid email or password');
    error.statusCode = 404;
    error.status = 'fail';
    return error;
  }
  return userExists;
}

const getAllUsers = async () => {
  try {
    const users = await User.find();
    return users;
  } catch {
    const error = new ApiError('Error during fetching users');
    error.statusCode = 500;
    error.status = 'fail';
    return error;
  }
}

const deleteUser = async (id: string) => {
  try {
    const user = await User.findByIdAndDelete(id);
    return user;
  } catch {
    const error = new ApiError('Error during user deletion');
    error.statusCode = 500;
    error.status = 'fail';
    return error;
  }
}


export {
  createUser,
  loginUser,
  getAllUsers,
  deleteUser
}