import mongoose, { Schema, Document } from 'mongoose';

interface IProduct extends Document {
  name: string;
  description: string;
  price: number;
  userId: string;
  createdAt: Date;
  updatedAt: Date;
}

const ProductSchema: Schema = new Schema({
  name: { type: String, required: true },
  description: { type: String, required: true },
  price: { type: Number, required: true },
  userId: { type: Schema.Types.ObjectId, ref: 'User', required: true },
  createdAt: { type: Date, default: Date.now },
  updatedAt: { type: Date, default: Date.now },
});

ProductSchema.pre('save', async function (next) {
  try {
    if (!this.name || !this.description || !this.price || !this.userId) {
      throw new Error('Missing required fields.');
    }

    const productExists = await mongoose.model<IProduct>('Product').exists({ name: this.name, userId: this.userId });
    if (productExists) {
      throw new Error('Product already exists.');
    }

    next();
  } catch (error) {
    next(error);
  }
});

const Product = mongoose.model<IProduct>('Product', ProductSchema);

ProductSchema.index({ name: 1, userId: 1 }, { unique: true });


export {
  Product,
  IProduct
}
